#!/usr/bin/python
__author__ = 'Jiri_Kulda'

import re
import shlex
import bkrdoc.analysis
import sys


class PhaseOutside:
    """Class for searching data outside of phases"""
    phase_name = ""
    statement_list = []
    variables = ""
    func_list = []

    def __init__(self):
        # self.parse_ref = parse_cmd
        self.phase_name = "Outside phase"
        self.statement_list = []
        self.variables = bkrdoc.analysis.TestVariables()
        self.func_list = []

    def setup_statement(self, line):
        self.statement_list.append(line)

    def search_data(self, variable_copy, function_copy):
        """
        This method searches variables in statements. Also when it finds function then this method
        created TestFunction object with function data.
        :param variable_copy: Copy of variables
        :param function_copy: Copy of functions
        """
        self.variables = variable_copy
        self.func_list = function_copy
        func = False
        for statement in self.statement_list:

            # These three conditions are here because of getting further
            # information from functions.
            if self.is_function(statement):
                func = True
                self.func_list.append(bkrdoc.analysis.TestFunction(statement[len("function")+1:]))

            elif func and not self.is_function_end(statement):
                self.func_list[-1].add_line(statement)

            elif func and self.is_function_end(statement):
                self.func_list[-1].add_line(statement)
                func = False

            else:
                # searching variables in statement line
                try:
                    read = shlex.shlex(statement)
                    member = read.get_token()
                    equal_to = read.get_token()

                    while equal_to:
                    # condition to handle assign to random value
                    # setting variable list
                        if equal_to == '=':
                        # These 7 lines are here for erasing comments and for reading whole line
                            pom_i = statement.find("=", len(member)) + 1
                            list_of_statement = shlex.split(statement[pom_i:], True, True)
                            value = ""
                            for value_member in list_of_statement:
                                if not value == "":
                                    value += " "
                                value += value_member

                            regular = re.compile("\"(/.*/)(.*)\"")
                            match = regular.match(value)
                            if match:
                                self.variables.add_variable(member, match.group(1) + match.group(2))
                                self.variables.add_keyword(match.group(2))
                            else:
                                self.variables.add_variable(member, value)

                        member = equal_to
                        equal_to = read.get_token()
                except ValueError as detail:
                    print("ERROR in line: " + str(statement))
                    print("With message: " + str(detail))

    def is_function(self, line):
        return line[0:len("function")] == "function"

    def is_function_end(self, line):
        """
        Test end of line of function in statement line
        :param line: statement line
        :return: true or false
        """
        if line[0:1] == "}":
            return True
        else:
            # This split for erasing comments on the end of line
            pom_list = shlex.split(line, True, True)
            if pom_list[-1][-1] == "}":
                return True
            else:
                return False


class PhaseContainer:
    """Class for storing information in test phase"""
    phase_name = ""
    statement_list = []
    doc_ref = ""
    variables = ""
    statement_classes = []
    documentation_units = []
    phase_documentation_information = []
    func_list = []
    generator_ref = ""

    def __init__(self, name):
        self.phase_name = name
        self.statement_list = []
        self.doc = []
        self.variables = bkrdoc.analysis.TestVariables()
        self.statement_classes = []
        self.documentation_units = []
        self.phase_documentation_information = []
        self.func_list = []
        self.generator_ref = ""

    def setup_statement(self, line):
        self.statement_list.append(line)

    def search_data(self, generator_ref, variable_copy, function_copy):
        """
        This method runs data searching in statement lines.
        :param generator_ref: parser object reference
        :param variable_copy: variable copy
        :param function_copy: function copy
        """
        self.func_list = function_copy
        self.variables = variable_copy
        self.generator_ref = generator_ref
        command_translator = bkrdoc.analysis.StatementDataSearcher(generator_ref, self)
        for statement in self.statement_list:
            try:
                self.statement_classes.append(command_translator.parse_command(statement))
            except ValueError:
                sys.stdout.write("********************************************")
                print("ERROR in line: " + str(statement))
                print(ValueError)
                print("********************************************")
            except SystemExit:
                sys.stdout.write("********************************************")
                print("ERROR in line: " + str(statement))
                print("Can be problem with variables substitutions")
                print("********************************************")

    def search_data_in_function(self, function):
        """
        Searching data in function object
        :param function: function object
        """
        command_translator = bkrdoc.analysis.StatementDataSearcher(self.generator_ref, self)
        function.data_list = []
        for statement in function.statement_list:
            try:
                function.add_data(command_translator.parse_command(statement))
            except ValueError:
                sys.stdout.write("********************************************")
                print("ERROR in line: " + str(statement))
                print(ValueError)
                print("********************************************")
            except SystemExit:
                sys.stdout.write("********************************************")
                print("ERROR in line: " + str(statement))
                print("Can be problem with variables substitutions")
                print("********************************************")

    def translate_data(self, parser_ref):
        """
        Translate data from argparse object to DocumentationInformation object
        :param parser_ref: parser reference
        """
        data_translator = bkrdoc.analysis.DocumentationTranslator(parser_ref)
        for data in self.statement_classes:
            if data.argname != "UNKNOWN":
                self.documentation_units.append(data_translator.translate_data(data))

    def generate_documentation(self):
        """
        Transforms DocumentationInformation into small classes using GetInformation
        """
        information_translator = bkrdoc.analysis.GetInformation()
        for information in self.documentation_units:
            if information:
                self.phase_documentation_information.append(information_translator.get_information_from_facts(information))

    def print_phase_documentation(self, cmd_options):
        """
        Prints nature language information
        :param cmd_options: possible command line options
        """
        self.print_phase_name_with_documentation_credibility()
        conditions = bkrdoc.analysis.ConditionsForCommands()

        for information in self.phase_documentation_information:
            if cmd_options.log_in or cmd_options.print_all:
                information.print_information()
            elif not conditions.is_rllog_command(information.get_command_name()):
                information.print_information()

    def print_phase_name_with_documentation_credibility(self):
        total_commands = len(self.statement_list)
        unknown_commands = total_commands - len(self.phase_documentation_information)
        inf = self.phase_name + " [ Unknown commands: " + str(unknown_commands) + ", Total: " + str(total_commands) \
                              + ", Documentation credibility: " + self.get_phase_credibility().get_credibility() + " ]"
        print(inf)

    def get_information_list(self):
        return self.phase_documentation_information

    def set_information_list(self, inf_list):
        """
        Sets whole information list. This method is called after solving knapsack problem.
        :param inf_list: list of information from finished knapsack algorithm
        """
        self.phase_documentation_information = inf_list

    def get_phase_weigh(self):
        """
        this method returns number of lines on which information will be printed

        :return: -//-
        """
        phase_weigh = 0
        for inf in self.phase_documentation_information:
            phase_weigh += inf.get_information_weigh()
        return phase_weigh

    def get_function_list(self):
        """
        :return: list of functions
        """
        return self.func_list

    def get_phase_credibility(self):
        Credibility = bkrdoc.analysis.credibility.DocumentationCredibility
        total_commands = len(self.statement_list)
        unknown_commands = total_commands - len(self.phase_documentation_information)
        percent_correct = Credibility.compute_percent_correct(unknown_commands, total_commands)
        return Credibility(percent_correct)



#!/usr/bin/python
__author__ = 'Jiri_Kulda'

class TestVariables:
    """Class contain variables from BeakerLib test"""
    variable_names_list = []
    variable_values_list = []

    keywords = []

    def __init__(self):
        self.variable_names_list = []
        self.variable_values_list = []

    def add_variable(self, name, value):
        if self.is_existing_variable(name):
            pom_pos = self.get_variable_position(name)
            self.variable_values_list[pom_pos] = value
        else:
            self.variable_names_list.append(name)
            self.variable_values_list.append(value)

    def add_keyword(self, keyword):
        if not self.is_existing_keyword(keyword):
            self.keywords.append(keyword)

    def is_existing_keyword(self, keyword):
        return keyword in self.keywords

    def is_existing_variable(self, name):
        return name in self.variable_names_list

    def get_variable_value(self, name):
        pos = self.get_variable_position(name)
        return self.variable_values_list[pos]

    def get_variable_position(self, name):
        i = 0
        for element in self.variable_names_list:
            if element == name:
                return i
            i += 1
        return -1

    def replace_variable_in_string(self, string):
        i = 0
        pom_str = string
        if len(self.variable_names_list):
            for element in self.variable_names_list:
                pom_str = pom_str.replace("$" + element, self.variable_values_list[i])
                i += 1
            return pom_str
        else:
            return string


__author__ = 'jkulda'
import argparse
from bkrdoc.analysis.documentation_generator import run_analysis_doc_generator
from bkrdoc.markup.tagged_generator import run_markup_doc_generator


def analysis_run(args):
    run_analysis_doc_generator(args)


def markup_run(args):
    run_markup_doc_generator(args)


#  ***************** MAIN ******************
def parse_cmd_arguments():
    """
    This method contains set upped argparse object to parse
    command line options and files

    :return: argparse object
    """
    parser = argparse.ArgumentParser(description='Director of bkrdoc generator type choosing')
    subparsers = parser.add_subparsers()
    parser.add_argument('files',
                        metavar='file',
                        type=str,
                        nargs='+',
                        help='script file')

    parser_analysis = subparsers.add_parser("analysis")
    parser_analysis.add_argument('-l', '--log',
                                 dest='log_in',
                                 action='store_true',
                                 default=False,
                                 help='Show log data if they are possible')
    parser_analysis.add_argument('-s', '--size',
                                 type=int,
                                 help="Size of output documentation in lines, default is 32 lines(A4 size) per"
                                      " documentation",
                                 default=32)
    parser_analysis.add_argument('--all', '--print-all',
                                 dest='print_all',
                                 action='store_true',
                                 default=False,
                                 help='Printing all possible data.')
    parser_analysis.set_defaults(func=analysis_run)

    parser_markup = subparsers.add_parser("markup")
    parser_markup.add_argument('-o', '--output',
                               dest='FILE_NAME',
                               type=str,
                               default=False,
                               help='Save documentation into file with specified name.')
    parser_markup.set_defaults(func=markup_run)
    parser_arg = parser.parse_args()

    try:
        parser_arg.func(parser_arg)
    except Exception as exc:
        print ("Error with function parsing. If this is a bug make an issue in github repo")
        print exc

if __name__ == "__main__":
    parse_cmd_arguments()



#!/usr/bin/python
__author__ = 'Jiri_Kulda'

import bkrdoc.analysis


class DocumentationTranslator:
    """Class making documentation information from argparse data.
    Generated information are focused on BeakerLib commands"""
    inf_ref = ""

    low = 1
    lowMedium = 2
    medium = 3
    high = 4
    highest = 5

    def __init__(self, generator_ref):
        self.generator_ref = generator_ref
        self.inf_ref = ""

    def translate_data(self, argparse_data):
        """
        This method translate argparse object into DocumentationInformation object
        :param argparse_data: argparse object
        :return: DocumentationInformation object
        """
        self.inf_ref = ""

        argname = argparse_data.argname
        condition = bkrdoc.analysis.ConditionsForCommands()

        if condition.is_rlrun_command(argname):
            self.set_rlrun_data(argparse_data)

        elif condition.is_rpm_command(argname):
            self.set_rpmcommand_data(argparse_data)

        elif condition.is_check_or_assert_mount(argname):
            self.set_check_or_assert_mount_data(argparse_data)

        elif condition.is_assert_command(argname):

            if condition.is_assert_grep(argname):
                self.set_assert_grep_data(argparse_data)

            elif condition.is_rlpass_or_rlfail_command(argname):
                self.set_rlpass_or_rlfail_data(argparse_data)

            elif condition.is_assert0(argname):
                self.set_rlassert0_data(argparse_data)

            elif condition.is_assert_comparasion(argname):
                self.set_assert_comparison_data(argparse_data)

            elif condition.is_assert_exists(argname):
                self.set_assert_exits_data(argparse_data)

            elif condition.is_assert_differ(argname):
                self.set_assert_differ_data(argparse_data)

            elif condition.is_assert_binary_origin(argname):
                self.set_assertbinaryorigin_data(argparse_data)

        elif condition.is_rlfilebackup_command(argname):
            self.set_rlfilebackup_data(argparse_data)

        elif condition.is_rlfilerestore_command(argname):
            self.set_rlfile_restore_data(argparse_data)

        elif condition.is_rlisrhel_or_rlisfedora_command(argname):
            self.set_isrhel_or_isfedora_data(argparse_data)

        elif condition.is_rlmount(argname):
            self.set_rlmount_data(argparse_data)

        elif condition.is_rlhash_or_rlunhash_command(argname):
            self.set_rlhash_or_rlunhash_data(argparse_data)

        elif condition.is_rllog_command(argname):
            self.set_rllog_data(argparse_data)

        elif condition.is_rldie_command(argname):
            self.set_rldie_data(argparse_data)

        elif condition.is_rlget_x_arch_command(argname):
            self.set_rlget_commands_data(argparse_data)

        elif condition.is_rlgetdistro_command(argname):
            self.set_rlget_commands_data(argparse_data)

        elif condition.is_rlgetphase_or_test_state_command(argname):
            self.set_rlget_commands_data(argparse_data)

        elif condition.is_rlreport_command(argname):
            self.set_rlreport_data(argparse_data)

        elif condition.is_rlwatchdog_command(argname):
            self.set_rlwatchdog_data(argparse_data)

        elif condition.is_rlbundlelogs_command(argname):
            self.set_rlbundlelogs_data(argparse_data)

        elif condition.is_rlservicexxx(argname):
            self.set_rlservicexxx_data(argparse_data)

        elif condition.is_sebooleanxxx_command(argname):
            self.set_sebooleanxxx_data(argparse_data)

        elif condition.is_rlshowrunningkernel_command(argname):
            self.set_rlshowrunningkernel_data()

        elif condition.is_get_or_check_makefile_requires(argname):
            self.set_rlget_or_rlcheck_makefilerequeries_data(argparse_data)

        elif condition.is_rlcleanup_apend_or_prepend_command(argname):
            self.set_rlcleanup_apend_or_prepend_data(argparse_data)

        elif condition.is_rlfilesubmit_command(argname):
            self.set_rlfilesubmit_data(argparse_data)

        elif condition.is_rlperftime_runsintime_command(argname):
            self.set_rlperftime_runsintime_data(argparse_data)

        elif condition.is_rlperftime_avgfromruns_command(argname):
            self.set_rlperftime_avgfromruns_data(argparse_data)

        elif condition.is_rlshowpackageversion_command(argname):
            self.set_rlshowpackageversion_data(argparse_data)

        elif condition.is_rljournalprint_command(argname):
            self.set_rljournalprint_data(argparse_data)

        elif condition.is_rlimport_command(argname):
            self.set_rlimport_data(argparse_data)

        elif condition.is_rlwaitfor_command(argname):
            self.set_rlwaitfor_data(argparse_data)

        elif condition.is_rlwaitforcmd_command(argname):
            self.set_rlwaitforcmd_data(argparse_data)

        elif condition.is_rlwaitforfile_command(argname):
            self.set_rlwaitforfile_data(argparse_data)

        elif condition.is_rlwaitforsocket_command(argname):
            self.set_rlwaitforsocket_data(argparse_data)

        elif condition.is_virtualxxx_command(argname):
            self.set_rlvirtualx_xxx_data(argparse_data)

        return self.inf_ref

    def set_rljournalprint_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.low
        subject = []
        param_option = []
        if argparse_data.argname == "rlJournalPrint":
            if len(argparse_data.type):
                subject.append(argparse_data.type)
            else:
                subject.append("xml")
        else:
            subject.append("text")
            if argparse_data.full_journal:
                param_option.append("additional information")

        topic_obj = bkrdoc.analysis.Topic("JOURNAL", subject)
        action = ["print"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlshowpackageversion_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        action = ["print"]
        subject = argparse_data.package
        topic_obj = bkrdoc.analysis.Topic("PACKAGE", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlfilesubmit_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = [argparse_data.path_to_file]
        print(argparse_data.required_name)
        if not len(argparse_data.s) and not len(argparse_data.required_name):
            subject.append('-')

        elif len(argparse_data.s) and argparse_data.required_name is None:
            subject.append(argparse_data.s)

        elif len(argparse_data.s) and argparse_data.required_name is not None and len(argparse_data.required_name):
            subject.append(argparse_data.s)
            subject.append(argparse_data.required_name)
        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        action = ["resolve"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlbundlelogs_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.low
        subject = argparse_data.file
        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        action = ["create"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rldie_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.low
        subject = [argparse_data.message]
        subject += argparse_data.file
        topic_obj = bkrdoc.analysis.Topic("MESSAGE", subject)
        action = ["create"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rllog_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.low
        subject = [argparse_data.message]
        topic_obj = bkrdoc.analysis.Topic("MESSAGE", subject)
        action = ["create"]
        param_option = []
        if argparse_data.logfile:
            param_option.append(argparse_data.logfile)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlshowrunningkernel_data(self):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        """
        importance = self.lowMedium
        topic_obj = bkrdoc.analysis.Topic("MESSAGE", ["kernel"])
        action = ["create"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation("rlShowRunningKernel", topic_obj, action, importance)

    def set_rlget_or_rlcheck_makefilerequeries_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        topic_obj = bkrdoc.analysis.Topic("FILE", ["makefile"])
        action = []
        if argparse_data.argname == "rlGetMakefileRequires":
            action.append("print")
        else:
            action.append("check")
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlget_commands_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        subject = []
        action = []
        if bkrdoc.analysis.ConditionsForCommands().is_rlgetphase_or_test_state_command(argparse_data.argname):
            if argparse_data.argname == "rlGetTestState":
                subject.append("test")
            else:
                subject.append("phase")
        elif bkrdoc.analysis.ConditionsForCommands().is_rlgetdistro_command(argparse_data.argname):
            if argparse_data.argname == "rlGetDistroRelease":
                subject.append("release")
            else:
                subject.append("variant")
        elif argparse_data.argname == "rlGetPrimaryArch":
            subject.append("primary")
        else:
            subject.append("secondary")
        topic_obj = bkrdoc.analysis.Topic("JOURNAL", subject)
        action.append("return")
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlwatchdog_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.highest
        subject = ["watchdog", argparse_data.command, argparse_data.timeout]
        param_option = []
        if argparse_data.signal:
            param_option.append(argparse_data.signal)
        topic_obj = bkrdoc.analysis.Topic("COMMAND", subject)
        action = ["run"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlreport_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        subject = [argparse_data.name, argparse_data.result]
        topic_obj = bkrdoc.analysis.Topic("JOURNAL", subject)
        action = ["report"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlrun_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.highest
        subject = [argparse_data.command, argparse_data.status]
        possible_beakerlib_command = self.get_argparse_of_command(argparse_data.command)

        if possible_beakerlib_command.argname == "UNKNOWN":
            param_option = []
            if argparse_data.l:
                param_option.append("l")
            elif argparse_data.c:
                param_option.append("c")
            elif argparse_data.t and argparse_data.s:
                param_option.append("s")
                param_option.append("t")
            elif argparse_data.t:
                param_option.append("t")
            elif argparse_data.s:
                param_option.append("s")
            topic_obj = bkrdoc.analysis.Topic("COMMAND", subject)
            action = ["run"]
            self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

        else:
            beakerlib_information_unit = self.translate_data(possible_beakerlib_command)
            beakerlib_information_unit.set_status(argparse_data.status)

    def get_argparse_of_command(self, command):
        """
        Gets argparse from command. This method is called in few methods. It depends
        on BeakerLib commands. If argparse of BeakerLib command contain command data,
        then this method will be used.
        :param command: command line
        :return: argparse object
        """
        pom_phase = bkrdoc.analysis.PhaseContainer("Helpful phase")
        return bkrdoc.analysis.StatementDataSearcher(self.generator_ref, pom_phase).parse_command(command)

    def set_rlvirtualx_xxx_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        subject = [argparse_data.name]
        action = []
        if argparse_data.argname == "rlVirtualXStop":
            action.append("kill")
        elif argparse_data.argname == "rlVirtualXStart":
            action.append("run")
        else:
            action.append("return")
        topic_obj = bkrdoc.analysis.Topic("SERVER", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlwaitfor_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = []
        if len(argparse_data.n):
            subject = argparse_data.n
        topic_obj = bkrdoc.analysis.Topic("COMMAND", subject)
        action = ["wait"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlwaitforsocket_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = [argparse_data.port_path]
        param_option = []
        if argparse_data.close:
            param_option.append("close")
        elif argparse_data.p:
            param_option.append("p")

        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        action = ["wait"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlwaitforfile_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = ["file", argparse_data.path]
        param_option = []
        if argparse_data.p:
            param_option.append(argparse_data.p)
        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        action = ["wait"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlwaitforcmd_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = ["cmd", argparse_data.command]
        param_option = ["", ""]
        if argparse_data.r:
            param_option[0] = argparse_data.r

        if argparse_data.p:
            param_option[1] = argparse_data.p

        topic_obj = bkrdoc.analysis.Topic("COMMAND", subject)
        action = ["wait"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlimport_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        subject = argparse_data.LIBRARY
        topic_obj = bkrdoc.analysis.Topic("PACKAGE", subject)
        action = ["import"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlperftime_runsintime_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = [argparse_data.command]
        param_option = [argparse_data.time]
        topic_obj = bkrdoc.analysis.Topic("COMMAND", subject)
        action = ["measures"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlperftime_avgfromruns_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = [argparse_data.command]
        topic_obj = bkrdoc.analysis.Topic("COMMAND", subject)
        action = ["measures"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlcleanup_apend_or_prepend_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        subject = []
        if argparse_data.argname == "rlCleanupAppend":
            subject.append("append")
        subject.append(argparse_data.string)
        topic_obj = bkrdoc.analysis.Topic("STRING", subject)
        action = ["create"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_sebooleanxxx_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        subject = []
        if argparse_data.argname == "rlSEBooleanOn":
            subject.append("on")
        elif argparse_data.argname == "rlSEBooleanOff":
            subject.append("off")
        subject += argparse_data.boolean
        topic_obj = bkrdoc.analysis.Topic("BOOLEAN", subject)
        action = ["set"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlservicexxx_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        subject = argparse_data.service
        action = []
        if argparse_data.argname == "rlServiceStart":
            action.append("run")
        elif argparse_data.argname == "rlServiceStop":
            action.append("kill")
        else:
            action.append("restore")
        topic_obj = bkrdoc.analysis.Topic("SERVICE", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlfile_restore_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        param_option = []
        if argparse_data.namespace:
            param_option.append(argparse_data.namespace)
        topic_obj = bkrdoc.analysis.Topic("FILE", [""])
        action = ["restore"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlfilebackup_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        param_option = []
        subject = argparse_data.file
        if argparse_data.namespace:
            param_option.append(argparse_data.namespace)

        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        action = ["backup"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_rlhash_or_rlunhash_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        param_option = []
        subject = []
        if argparse_data.stdin:
            subject.append(argparse_data.stdin)
        else:
            subject.append(argparse_data.STRING)
        action = []
        if argparse_data.argname == "rlUnhash" or argparse_data.decode:
            action.append("unhash")
        else:
            action.append("hash")
        if argparse_data.algorithm:
            param_option.append(argparse_data.algorithm)
        topic_obj = bkrdoc.analysis.Topic("STRING", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_check_or_assert_mount_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = [argparse_data.mountpoint]
        action = []
        if argparse_data.argname == "rlCheckMount":
            action.append("check")
        else:
            action.append("exists")
        if argparse_data.server and argparse_data.mountpoint:
            subject.append(argparse_data.server)
        topic_obj = bkrdoc.analysis.Topic("MOUNTPOINT", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlmount_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.lowMedium
        subject = [argparse_data.mountpoint, argparse_data.server]
        topic_obj = bkrdoc.analysis.Topic("MOUNTPOINT", subject)
        action = ["create"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_assertbinaryorigin_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        subject = [argparse_data.binary]
        subject += argparse_data.package
        topic_obj = bkrdoc.analysis.Topic("PACKAGE", subject)
        action = ["owned by"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rpmcommand_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        subject = []
        action = []
        subject.append(argparse_data.name)
        if argparse_data.argname == "rlCheckRpm":
            action.append("check")
        elif argparse_data.argname == "rlAssertRpm":
            action.append("exists")
            if argparse_data.all:
                subject.append("all")
        else:
            action.append("not exists")
        topic_obj = bkrdoc.analysis.Topic("PACKAGE", subject)
        param_option = []
        if argparse_data.version or argparse_data.release or \
                argparse_data.arch:
            if argparse_data.version:
                param_option.append(argparse_data.version)

            if argparse_data.release:
                param_option.append(argparse_data.release)

            if argparse_data.arch:
                param_option.append(argparse_data.arch)

        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))

    def set_isrhel_or_isfedora_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.medium
        action = []
        subject = []
        if argparse_data.argname == "rlIsRHEL":
            action.append("RHEL")
        else:
            action.append("Fedora")
        if len(argparse_data.type):
            subject = argparse_data.type
        topic_obj = bkrdoc.analysis.Topic("SYSTEM", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_assert_differ_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        action = []
        if argparse_data.argname == "rlAssertDiffer":
            action.append("differ")
        else:
            action.append("not differ")
        subject = [argparse_data.file1, argparse_data.file2]
        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_assert_exits_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        subject = [argparse_data.file_directory]
        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        action = []
        if argparse_data.argname == "rlAssertExists":
            action.append("exists")
        else:
            action.append("not exists")
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_assert_comparison_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        action = []
        subject = [argparse_data.value1, argparse_data.value2]
        if argparse_data.argname == "rlAssertEquals":
            action.append("equal")
        elif argparse_data.argname == "rlAssertNotEquals":
            action.append("not equal")
        elif argparse_data.argname == "rlAssertGreater":
            action.append("greater")
        else:
            action.append("greater or equal")
        topic_obj = bkrdoc.analysis.Topic("VALUE", subject)
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlassert0_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        topic_obj = bkrdoc.analysis.Topic("VALUE", [argparse_data.value])
        action = ["check"]
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance)

    def set_rlpass_or_rlfail_data(self, argparse_data):
        pass

    def set_assert_grep_data(self, argparse_data):
        """
        Sets DocumentationInformation object to specified BeakerLib command
        :param argparse_data: argparse object
        """
        importance = self.high
        subject = [argparse_data.file, argparse_data.pattern]
        topic_obj = bkrdoc.analysis.Topic("FILE", subject)
        action = []
        if argparse_data.argname == "rlAssertGrep":
            action.append("contain")
        else:
            action.append("not contain")
        param_option = []
        if argparse_data.text_in:
            param_option.append("text_in")
        elif argparse_data.moin_in:
            param_option.append("moin_in")
        elif argparse_data.out_in:
            param_option.append("out_in")
        self.inf_ref = bkrdoc.analysis.DocumentationInformation(argparse_data.argname, topic_obj, action, importance, bkrdoc.analysis.Option(param_option))


__author__ = 'jkulda'

import argparse
import sys
from bkrdoc.markup.tagged_parser import Parser
from bkrdoc.markup.data_containers import TaggedCommentContainer


class Generator(object):

    phases = []

    def __init__(self, file_in):
        if file_in[(len(file_in) - 3):len(file_in)] == ".sh":
            try:
                self.filename = file_in
                with open(file_in, "r") as input_file:
                    self.file_string = input_file.read()
                    self.parser_ref = ""
                    self.phases = []

            except IOError:
                sys.stderr.write("ERROR: Fail to open file: " + file_in + "\n")
                sys.exit(1)
        else:
            print ("ERROR: Not a script file. (.sh)")
            sys.exit(1)

    def parse_file(self):
        self.parser_ref = Parser()
        self.parser_ref.parse_file_by_lines(self.file_string)
        self.phases = self.parser_ref.get_phases()

    def comments_set_up(self):
        for phase in self.phases:
            phase.comments_set_up()

    def get_title_data(self):
        self.phases[0].search_for_title_data()

    def erase_title_data(self):
        help_comment = TaggedCommentContainer("")
        help_comment.erase_known_tags()

    def get_documentation_title_doc(self):
        self.get_title_data()
        title_data = self.phases[0].comments_list[0].get_title_data()
        documentation = "Description: {0}\n".format(title_data["description"])
        documentation += "Author: {0}\n".format(title_data["author"])
        if title_data["key"] and title_data["keywords"]:
            documentation += "Keywords: {0}, {1}".format(title_data["keywords"], title_data["key"])
        elif title_data["key"] and not title_data["keywords"]:
            documentation += "Keywords: {0}".format(title_data["key"])
        else:
            documentation += "Keywords: {0}".format(title_data["keywords"])
        # need to erase shared title data
        self.erase_title_data()
        return documentation

    def get_phase_data_documentation(self):
        documentation = "Phases:\n"
        pom_doc = ""
        for phase in self.phases:
            pom_doc += phase.print_documentation()
            phases_doc_split = pom_doc.strip().split("\n")
            if len(phases_doc_split) > 1:
                documentation += pom_doc
            pom_doc = ""
        return documentation

    def print_additional_container_data(self, name, additional_containers):
        documentation = ""
        if len(additional_containers):
            documentation += "  {0}:\n".format(name)
            for container in additional_containers:
                documentation += container.get_additional_phase_data()
                if self.is_condition_container(container):
                    for elif_member in container.elif_parts:
                        documentation += elif_member.get_additional_phase_data()
                    documentation += "{0}fi\n".format("    ")
                if len(additional_containers) > 1:
                    documentation += "\n"
            documentation += "\n"
        return documentation

    def is_condition_container(self, container):
        return type(container).__name__ == "ConditionContainer"

    def generate_additional_info(self):
        documentation = "Additional information:\n"
        func = []
        loop = []
        cond = []
        for phase in self.phases:
            func, loop, cond = phase.get_additional_containers(func, loop, cond)

        documentation += self.print_additional_container_data("Functions", func)
        documentation += self.print_additional_container_data("Loops", loop)
        documentation += self.print_additional_container_data("Conditions", cond)
        return documentation

    def get_documentation(self):
        documentation = self.get_documentation_title_doc()
        documentation += "\n\n"
        documentation += self.get_phase_data_documentation()
        documentation += "\n"
        documentation += self.generate_additional_info()
        return documentation

    def show_documentation_data(self, parsed_arg):
        documentation = self.get_documentation()
        if parsed_arg:
            file_out = open(parsed_arg, "w")
            file_out.write(documentation)
        else:
            sys.stdout.write(documentation)


# !!!!!!!!!!MAIN!!!!!!!!!!!!!!!!!!!
def set_cmd_arguments():
    # Parse of arguments
    parser = argparse.ArgumentParser(description='Documentation generator which creates documentation from tagged '
                                                 'documentation comments')
    parser.add_argument('files',
                        metavar='file',
                        type=str,
                        nargs='+',
                        help='script file')

    parser.add_argument('-o', '--output',
                        dest='FILE_NAME',
                        type=str,
                        default=False,
                        help='Save documentation into file with specified name.')
    return parser.parse_args()


def run_markup_doc_generator(parsed_arg):
    # cycle of script files to be transformed to documentation
    for file_in_cmd in parsed_arg.files:
        part = Generator(file_in_cmd)
        part.parse_file()
        part.comments_set_up()
        part.show_documentation_data(parsed_arg.FILE_NAME)


if __name__ == "__main__":
    CMD_args = set_cmd_arguments()
    run_markup_doc_generator(CMD_args)




from __future__ import division  # floating-point division


class DocumentationCredibility:
    """
    DocumentationCredibility class provides simple key(0-5 (LOW to HIGH)) to credibility conversion
    based on Unknown to Total commands ratio.
    :param credibility: credibility computed from number of unknown vs. total commands provided to the constructor
    :param percent_correct: % of known commands
    :param credibilityTable: dictionary-like key-to-credibility conversion
    """
    credibility = ""
    percent_correct = 0.0

    credibilityTable = {0: 'None',
                        1: 'Very low',
                        2: 'Low',
                        3: 'Medium',
                        4: 'High',
                        5: 'Very high'}

    def __init__(self, percent_correct):
        self.percent_correct = percent_correct
        self.credibility = self.compute_credibility()

    def lookup_credibility(self, key):
        return self.credibilityTable.get(key, "unknown")

    def compute_credibility_key(self, percent_correct):
        """
        Computes credibility key by 20% increments (known/total commands ratio), starting from zero.
        :return: key for credibility computation (0-5)
        """
        if self.is_zero(percent_correct):
            return 0
        elif self.all_correct(percent_correct):
            return 5
        else:
            return (percent_correct // 20) + 1

    def compute_credibility(self):
        key = self.compute_credibility_key(self.get_percent_correct())
        return self.lookup_credibility(key)

    def get_credibility(self):
        return self.credibility

    def get_percent_correct(self):
        return self.percent_correct

    @staticmethod
    def is_zero(number):
        return round(number) == 0

    @staticmethod
    def all_correct(number):
        return round(number) == 100

    @staticmethod
    def compute_percent_correct(unknown_commands, total_commands):
        return 100 if (total_commands == 0) else (1 - (unknown_commands / total_commands)) * 100



#!/usr/bin/python
__author__ = 'Jiri_Kulda'


class Option(object):
    """
    Option class which is consist of option and status of BeakerLib command
    :param option_data: BeakerLib command options data
    :param status_data: BeakerLib command future exit status
    """
    option = []

    status = []

    def __init__(self, option_data=None, status_data="-"):
        if option_data is None:
            self.option = []
        else:
            self.option = option_data
        self.status = status_data

    def get_option(self):
        return self.option

    def get_status(self):
        return self.status

    def set_status(self, new_status):
        self.status = new_status



__author__ = 'jkulda'

import shlex


class SimpleContainer(object):

    comments_list = []
    statement_list = []
    common_comments = []
    name = ""

    def add_comment(self, comment_container):
        self.comments_list.append(comment_container)

    def add_statement_line(self, line):
        self.statement_list.append(line)

    def add_common_comment(self, common_comment):
        self.common_comments.append(common_comment)

    def get_statement_list(self):
        pom_list = []
        for member in self.statement_list:
            if self.is_simple_container_instance(member):
                if self.is_condition_container(member):
                    pom_list += member.get_statement_list()
                    for elif_member in member.elif_parts:
                        if self.is_simple_container_instance(elif_member):
                            pom_list += elif_member.get_statement_list()
                else:
                    pom_list += member.get_statement_list()
            else:
                pom_list.append(member)
        return pom_list

    def get_comments_list(self):
        pom_list = []
        for member in self.comments_list:
            if self.is_simple_container_instance(member):
                if self.is_condition_container(member):
                    pom_list += member.get_comments_list()
                    for elif_member in member.elif_parts:
                        if self.is_simple_container_instance(elif_member):
                            pom_list += elif_member.get_comments_list()
                else:
                    pom_list += member.get_comments_list()
            else:
                pom_list.append(member.get_comments())
        return pom_list

    def comments_set_up(self):
        for comment in self.comments_list:
            if self.is_simple_container_instance(comment):
                if self.is_condition_container(comment):
                    comment.comments_set_up()
                    for elif_comment in comment.elif_parts:
                        if self.is_simple_container_instance(elif_comment):
                            elif_comment.comments_set_up()
                else:
                    comment.comments_set_up()
            else:
                try:
                    comment.search_for_tags()
                except UnknownTagException as detail:
                    print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
                    print("Unknown Tag Exception: \"{0}\"".format(detail))
                    print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")

    def get_additional_containers(self, func, loop, cond):
        for comment in self.comments_list:
            if self.is_simple_container_instance(comment):
                # print comment
                if self.is_function_container(comment):
                    func.append(comment)
                    func, loop, cond = comment.get_additional_containers(func, loop, cond)
                elif self.is_loop_container(comment):
                    loop.append(comment)
                    func, loop, cond = comment.get_additional_containers(func, loop, cond)
                elif self.is_condition_container(comment):
                    cond.append(comment)
                    func, loop, cond = comment.get_additional_containers(func, loop, cond)
                    for element in comment.elif_parts:
                        if self.is_simple_container_instance(element):
                            func, loop, cond = element.get_additional_containers(func, loop, cond)
        return func, loop, cond

    def print_documentation(self):
        documentation = ""
        if not self.is_comment_list_empty():
            if self.is_test_phase_container():
                documentation += "  {0} {1}\n".format(self.name, self.name_comment)
            else:
                documentation += "  {0}\n".format(self.name)
            if self.is_outside_phase_container():
                documentation += self.print_comments_with_offset("    ")
            else:
                documentation += self.print_comments_with_offset("      ")
            documentation += "\n"
        return documentation

    def get_additional_phase_data(self):
        pass

    def is_comment_list_empty(self):
        if not self.comments_list:
            return False
        else:
            return self.search_for_emptiness(self.comments_list)

    def print_comments_with_offset(self, offset):
        documentation = ""
        documentation_tag = self.get_name()

        if not self.is_comment_list_empty():
            first = True
            for comment in self.comments_list:
                if self.is_simple_container_instance(comment):
                    # print "LOL {0}".format(documentation_tag)
                    # print self.comments_list
                    if first:
                        # print "???..."
                        documentation += "{0}{1}:\n".format(offset[2:], documentation_tag)
                        first = False
                    if self.is_condition_container(comment):
                        documentation += comment.print_comments_with_offset(offset + "  ")
                        for elif_comment in comment.elif_parts:
                            if self.is_simple_container_instance(elif_comment):
                                documentation += elif_comment.print_comments_with_offset(offset + "  ")
                    else:
                        # print "???"
                        documentation += comment.print_comments_with_offset(offset + "  ")
                else:
                    if first:
                        if self.is_setup_test_cleanup_tag(documentation_tag):
                            documentation += comment.print_data(offset[2:], "")
                            # print "{0} !!!!!!! {1}".format(documentation_tag, type(comment).__name__)
                        elif self.is_condition_container(self):
                            # print "{0} +++++++++++++++++ {1} in {2}".format(documentation_tag, type(comment).__name__, type(self).__name__)
                            if not self.search_for_emptiness(self.comments_list) and \
                                    self.search_for_emptiness(self.comments_list[1:]) and \
                                    self.comments_list[0].is_description_comment():
                                if self.search_for_emptiness(self.elif_parts):
                                    documentation += comment.print_data(offset[2:], "")
                                else:
                                    documentation += comment.print_data(offset, documentation_tag)
                            else:
                                documentation += comment.print_data(offset, documentation_tag)
                        elif self.is_loop_container(self):
                            if self.comments_list[0].is_description_comment() and self.search_for_emptiness(self.comments_list[1:]):
                                documentation += comment.print_data(offset[2:], "")
                            else:
                                # print "{0} /////////////////// {1} in {2}".format(documentation_tag, type(comment).__name__, type(self).__name__)
                                documentation += comment.print_data(offset, documentation_tag)
                        else:
                            documentation += comment.print_data(offset, documentation_tag)
                            # print "{0} --------- {1} in {2}".format(documentation_tag, type(comment).__name__, type(self).__name__)
                        first = False
                    else:
                        # print "{0} :::::::: {1}".format(documentation_tag, type(comment).__name__)
                        documentation += comment.print_data(offset, "")
            return documentation
        return ""

    def get_name(self):
        pass

    def search_for_emptiness(self, data):
        empty = True
        for member in data:
            if self.is_simple_container_instance(member):
                if not member.is_comment_list_empty():
                    empty = False
            elif self.is_tagged_comment_container(member):
                return False
        return empty

    def is_simple_container_instance(self, container):
        container_names = ["LoopContainer", "FunctionContainer", "ConditionContainer"]
        return type(container).__name__ in container_names

    def is_test_phase_container(self):
        return type(self).__name__ == "TestPhaseContainer"

    def is_outside_phase_container(self):
        return type(self).__name__ == "PhaseOutsideContainer"

    def is_loop_container(self, container):
        return type(container).__name__ == "LoopContainer"

    def is_function_container(self, container):
        return type(container).__name__ == "FunctionContainer"

    def is_condition_container(self, container):
        return type(container).__name__ == "ConditionContainer"

    def is_setup_test_cleanup_tag(self, tag):
        tags = ["Cleanup", "Setup", "Test"]
        return tag in tags

    def is_tagged_comment_container(self, container):
        return type(container).__name__ == "TaggedCommentContainer"


class PhaseOutsideContainer(SimpleContainer):

    name = "Outside Phase"

    def __init__(self):
        self.comments_list = []
        self.statement_list = []
        self.common_comments = []

    def search_for_title_data(self):
        new_coment = TaggedCommentContainer("")
        for common_comment in self.common_comments:
            if self.is_word_in_line(common_comment, "keywords:"):
                index = self.get_word_index_in_line(common_comment, "keywords:")
                data = new_coment.set_documentation_comment(common_comment[index + 1:])
                if new_coment.is_know_tag_empty("keywords"):
                    new_coment.known_tags["keywords"] = data
                else:
                    new_coment.known_tags["keywords"] += ", {0}".format(data)

            elif self.is_word_in_line(common_comment, "description:"):
                index = self.get_word_index_in_line(common_comment, "description:")
                data = new_coment.set_documentation_comment(common_comment[index + 1:])
                if new_coment.is_know_tag_empty("description"):
                    new_coment.known_tags["description"] = data
                else:
                    new_coment.known_tags["description"] += ", {0}".format(data)

            elif self.is_word_in_line(common_comment, "author:"):
                index = self.get_word_index_in_line(common_comment, "author:")
                data = new_coment.set_documentation_comment(common_comment[index + 1:])
                if new_coment.is_know_tag_empty("author"):
                    new_coment.known_tags["author"] = data
                else:
                    new_coment.known_tags["author"] += ", {0}".format(data)
        self.comments_list.insert(0, new_coment)
        new_coment = ""

    def get_name(self):
        return self.name

    def is_word_in_line(self, splitted_line, word):
        splitted_line = [element.lower() for element in splitted_line]
        return word in splitted_line

    def get_word_index_in_line(self, splitted_line, word):
        splitted_line = [element.lower() for element in splitted_line]
        try:
            return splitted_line.index(word)
        except ValueError:
            return 0


class TestPhaseContainer(SimpleContainer):

    def __init__(self):
        self.name = ""
        self.name_comment = ""
        self.comments_list = []
        self.statement_list = []
        self.common_comments = []

    def get_name(self):
        return self.name

    def add_statement_line(self, line):
        if self.is_name_empty() and len(self.statement_list) == 0:
            self.set_test_phase_name(line)
        else:
            self.statement_list.append(line)

    def is_name_empty(self):
        return self.name == ""

    def set_test_phase_name(self, line):
        line = shlex.split(line, posix=False)
        self.name = line[0][len("rlPhaseStart"):]
        if len(line) > 1:
            self.name_comment = line[1]


class ConditionContainer(SimpleContainer):
    condition_tag = "condition"
    elif_parts = []

    def __init__(self):
        self.comments_list = []
        self.statement_list = []
        self.common_comments = []
        self.elif_parts = []

    def get_additional_phase_data(self):
        offset = "    "
        documentation = "{0}{1}\n".format(offset, self.statement_list[0])
        for comment in self.comments_list:
            if not self.is_simple_container_instance(comment):
                documentation += comment.print_doc_comments_with_offset(offset + "  ")
        return documentation

    def add_elif_part(self, container):
        self.elif_parts.append(container)

    def get_name(self):
        if self.is_elif_in_line(self.statement_list[0]):
            return "elif"
        elif self.is_else_in_line(self.statement_list[0]):
            return "else"
        elif self.is_if_in_line(self.statement_list[0]):
            return "if"
        return self.condition_tag

    def is_elif_in_line(self, line):
        return line.find("elif") > -1

    def is_else_in_line(self, line):
        return line.find("else", 0, 5) > -1

    def is_if_in_line(self, line):
        return line.find("if", 0, 3) > -1

    def is_comment_list_empty(self):
        if not self.comments_list and not self.elif_parts:
            return True
        elif not self.comments_list and self.elif_parts:
            return self.search_for_emptiness(self.elif_parts)
        elif self.comments_list and not self.elif_parts:
            return self.search_for_emptiness(self.comments_list)
        else:
            empty = True
            if not self.search_for_emptiness(self.comments_list):
                empty = False
            if not self.search_for_emptiness(self.elif_parts):
                empty = False
            return empty


class FunctionContainer(SimpleContainer):
    function_tag = "function"

    def __init__(self):
        self.comments_list = []
        self.statement_list = []
        self.common_comments = []

    def get_additional_phase_data(self):
        offset = "    "
        documentation = "{0}{1}\n".format(offset, self.statement_list[0])
        documentation += offset + "{\n"
        for comment in self.comments_list:
            if not self.is_simple_container_instance(comment):
                documentation += comment.print_doc_comments_with_offset(offset + "  ")
        documentation += offset + "}\n"
        return documentation

    def get_name(self):
        return self.function_tag


class LoopContainer(SimpleContainer):
    loop_tag = "loop"

    def __init__(self):
        self.comments_list = []
        self.statement_list = []
        self.common_comments = []

    def get_additional_phase_data(self):
        offset = "    "
        documentation = "{0}{1}\n".format(offset, self.statement_list[0])
        documentation += "{0}do\n".format(offset)
        for comment in self.comments_list:
            if not self.is_simple_container_instance(comment):
                documentation += comment.print_doc_comments_with_offset(offset + "  ")
        documentation += "{0}done\n".format(offset)
        return documentation

    def get_name(self):
        if self.is_for_in_line(self.statement_list[0]):
            return "for"
        elif self.is_while_in_line(self.statement_list[0]):
            return "while"
        return self.loop_tag

    def is_for_in_line(self, line):
        return line.find("for", 0, 4) > -1

    def is_while_in_line(self, line):
        return line.find("while", 0, 6) > -1

    def is_do_while_in_line(self, line):
        return line.find("do", 0, 3) > -1

    def is_comment_list_empty(self):
        empty = True
        if not self.comments_list:
            return True
        else:
            return self.search_for_emptiness(self.comments_list)


class TaggedCommentContainer(object):

    known_tags = {"keywords": "", "key": "", "author": "", "description": ""}
    condition_tags = []
    comments = []
    tagged_line = ""
    documentation_comments = []

    def __init__(self, first_comment):
        self.comments = [first_comment]
        self.tagged_line = ""
        self.condition_tags = []
        self.documentation_comments = []

    def add_comment(self, comment):
        self.comments.append(comment)

    def add_tagged_line(self, line):
        self.tagged_line = line

    def get_comments(self):
        return self.comments

    def add_condition_tag(self, given_tag):
        self.condition_tags.append(given_tag)

    def erase_known_tags(self):
        self.known_tags["keywords"] = ""
        self.known_tags["key"] = ""
        self.known_tags["author"] = ""
        self.known_tags["description"] = ""

    def print_doc_comments_with_offset(self, offset):
        documentation = ""
        for comment in self.documentation_comments:
            documentation += "{0}{1}\n".format(offset, comment)
        return documentation

    def print_data(self, offset, documentation_tag):
        first = True
        documentation = ""
        #print "tag{0}".format(documentation_tag)
        for comment in self.documentation_comments:
            #print "Comment {0} with tagged line {1}".format(comment, self.tagged_line)
            if self.is_code_tag():
                if documentation_tag:
                    documentation += "{0}{1}: \n".format(offset[2:], documentation_tag)
                documentation += "{0}code: {1}\n".format(offset, comment)
            elif not self.is_splitted_line_empty(self.tagged_line):
                if documentation_tag and first:
                    if not self.is_function_loop_condition_line(self.tagged_line):
                        documentation += "{0}{1}: \n".format(offset[2:], documentation_tag)
                        documentation += "{0}{1}\n".format(offset, comment)
                    else:
                        documentation += "{0}{1}: {2}\n".format(offset[2:], documentation_tag, comment)
                    first = False

                elif self.is_function_loop_condition_line(self.tagged_line) and not first:
                    documentation += "{0}- {1}\n".format(offset[1:], comment)

                else:
                    #print "Comment {0} with tagged line {1}".format(comment, self.tagged_line)
                    #print "first {0}".format(first)
                    #print "documentation_tag {0}".format(documentation_tag)
                    documentation += "{0}{1}\n".format(offset, comment)

            else:
                documentation += "{0}{1}\n".format(offset, comment)
        return documentation

    def is_code_tag(self):
        if not self.is_splitted_line_empty(self.condition_tags):
            return self.condition_tags[0] == "code"
        return False

    def get_title_data(self):
        return self.known_tags

    def search_for_tags(self):
        for comment in self.comments:
            if self.get_tag_in_line(comment):
                found_tag = self.get_tag_in_line(comment)[0]
                if self.is_known_tag(self.get_tag_from_word(found_tag)):
                    known_tags_data = self.set_documentation_comment(comment[comment.index(found_tag) + 1:])
                    if self.is_know_tag_empty(self.get_tag_from_word(found_tag)):
                        self.known_tags[self.get_tag_from_word(found_tag)] = known_tags_data
                    else:
                        self.known_tags[self.get_tag_from_word(found_tag)] += ", {0}".format(known_tags_data)
                    # print self.known_tags
                    # print self.known_tags[self.get_tag_from_word(found_tag)]
                else:
                    raise UnknownTagException("Not supported tag: {0}. If it's needed write an e-mail to "
                                              "jkulda@redhat.com.".format(self.get_tag_from_word(found_tag)))
            else:
                erased_comment_line = self.erase_comments_start_tags(comment)
                if self.is_code_tag():
                    erased_comment_line = self.erase_comments_start_tags(self.tagged_line)
                self.documentation_comments.append(self.set_documentation_comment(erased_comment_line))
                # print self.documentation_comments

    def is_know_tag_empty(self, key):
        return self.known_tags[key] == ""

    def is_splitted_line_empty(self, splitted_line):
        return len(splitted_line) == 0

    def get_tag_in_line(self, splitted_line):
        return [word for word in splitted_line if ((word.startswith("@") and len(word) > 1) or word.startswith("#@@"))]

    def is_known_tag(self, tag):
        return tag in self.known_tags.keys()

    def erase_comments_start_tags(self, splitted_line):
        if splitted_line[0] == "#@":
            return splitted_line[1:]
        elif len(splitted_line) > 1 and splitted_line[0] == "#" and splitted_line[1] == "@":
            return splitted_line[2:]
        elif splitted_line[0].startswith("#@"):
            return [splitted_line[0][2:]] + splitted_line[1:]
        elif splitted_line[0].startswith("#"):
            if splitted_line[0] == "#":
                return splitted_line[1:]
            else:
                return [splitted_line[0][1:]] + splitted_line[1:]
        elif splitted_line[-1] == "#@":
            return splitted_line[0:-1]
        elif splitted_line[-1].endswith("#@"):
            return splitted_line[0:-2] + [splitted_line[-1][0:-3]]
        else:
            return splitted_line

    def set_documentation_comment(self, splitted_line):
        pom_line = splitted_line[0]
        for member in splitted_line[1:]:
            pom_line += " {0}".format(member)
        return pom_line

    def get_tag_from_word(self, word):
        if word.startswith("@"):
            return word[1:]
        else:
            return word[3:]

    def is_phase_start_xxx(self, splitted_line):
        phase_list = ["rlPhaseStartSetup", "rlPhaseStartCleanup", "rlPhaseStartTest"]
        if len(splitted_line) > 0:
            return splitted_line[0] in phase_list
        return False

    def is_condition_line(self, line):
        condition_line = ["if", "elif", "else"]
        return line[0] in condition_line

    def is_loop_line(self, line):
        loop_list = ["for", "while", "until"]
        return line[0] in loop_list

    def is_function_line(self, splitted_line):
        return splitted_line[0] == "function"

    def is_function_loop_condition_line(self, splitted_line):
        pom_list = ["if", "elif", "else", "for", "while", "until", "function"]
        return splitted_line[0] in pom_list

    def is_description_comment(self):
        return self.is_function_loop_condition_line(self.tagged_line)


class UnknownTagException(Exception):
    pass




#!/usr/bin/python
__author__ = 'Jiri_Kulda'

from bkrdoc.analysis import Option


class DocumentationInformation(object):
    """
    This class contains data to describe every BeakerLib command
    :param cmd_name: Command name
    :param topic_object: Instance of Topic class
    :param action: BeakerLib command action
    :param importance: BeakerLib command importance
    :param options: Instance of Option class
    """
    command_name = ""

    topic = ""

    options = Option

    action = []

    importance = ""

    def __init__(self, cmd_name, topic_object, action, importance, options=None):
        if options is None:
            self.options = Option()
        else:
            self.options = options
        self.command_name = cmd_name
        self.topic = topic_object
        self.action = action
        self.importance = importance

    def get_topic(self):
        return self.topic.get_topic()

    def get_topic_subject(self):
        return self.topic.get_subject()

    def get_action(self):
        return self.action

    def get_importance(self):
        return self.importance

    def get_status(self):
        return self.options.get_status()

    def get_option(self):
        return self.options.get_option()

    def set_status(self, status):
        self.options.set_status(status)

    def get_command_name(self):
        return self.command_name




__author__ = 'jkulda'

import unittest
from bkrdoc.markup.tagged_generator import Generator


class TaggedGeneratorTests(unittest.TestCase):

    def test_phases(self):
        generator = Generator("./examples/markup/abrt-auto-reporting-sanity-test.sh")
        generator.parse_file()
        parser = generator.parser_ref
        self.assertEqual(len(parser.phases), 19)
        self.assertEqual(type(parser.phases[0]).__name__,  "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[1]).__name__,  "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[2]).__name__,  "TestPhaseContainer")
        self.assertEqual(type(parser.phases[3]).__name__,  "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[4]).__name__,  "TestPhaseContainer")
        self.assertEqual(type(parser.phases[5]).__name__,  "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[6]).__name__,  "TestPhaseContainer")
        self.assertEqual(type(parser.phases[7]).__name__,  "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[8]).__name__,  "TestPhaseContainer")
        self.assertEqual(type(parser.phases[9]).__name__,  "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[10]).__name__, "TestPhaseContainer")
        self.assertEqual(type(parser.phases[11]).__name__, "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[12]).__name__, "TestPhaseContainer")
        self.assertEqual(type(parser.phases[13]).__name__, "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[14]).__name__, "TestPhaseContainer")
        self.assertEqual(type(parser.phases[15]).__name__, "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[16]).__name__, "TestPhaseContainer")
        self.assertEqual(type(parser.phases[17]).__name__, "PhaseOutsideContainer")
        self.assertEqual(type(parser.phases[18]).__name__, "PhaseOutsideContainer")

    def test_phases_statement(self):
        generator = Generator("./examples/markup/abrt-auto-reporting-sanity-test.sh")
        generator.parse_file()
        parser = generator.parser_ref

        self.assertEqual(parser.phases[0].get_statement_list(), ['. /usr/share/beakerlib/beakerlib.sh',
                                                                 '. ../aux/lib.sh', 'TEST="abrt-auto-reporting-sanity"',
                                                                 'PACKAGE="abrt"', 'function get_configured_value()',
                                                                 '{', 'VALUE=`grep "^AutoreportingEnabled" '
                                                                      '/etc/abrt/abrt.conf | tr -d " " | cut -f2 '
                                                                      '-d "="`', 'echo $VALUE', 'case "$VALUE" in',
                                                                 '[yY][eE][sS]|"_")', 'export CONF_VALUE="enabled"',
                                                                 ';;', '[nN][oO])', 'export CONF_VALUE="disabled"',
                                                                 ';;', '*)', 'echo "Unknown option value"',
                                                                 'export CONF_VALUE="disabled"', ';;', 'esac'])
        self.assertEqual(parser.phases[2].statement_list, ['TmpDir=$(mktemp -d)', 'pushd $TmpDir'])
        self.assertEqual(parser.phases[4].statement_list, ['rlRun "abrt-auto-reporting --help" 0',
                                                           'rlRun "abrt-auto-reporting --help 2>&1 | grep \'Usage: '
                                                           'abrt-auto-reporting\'"'])
        self.assertEqual(parser.phases[6].statement_list, ['rlRun "abrt-auto-reporting"', 'get_configured_value',
                                                           'rlAssertEquals "Reads the configuration" "_$(abrt-auto-reporting)" "_$CONF_VALUE"'])
        self.assertEqual(parser.phases[8].statement_list, ['rlRun "abrt-auto-reporting enabled"', 'get_configured_value',
                                                           'rlAssertEquals "Saves the configuration" "_enabled" "_$CONF_VALUE"', 'rlAssertEquals "Reads the configuration" "_enabled" "_$(abrt-auto-reporting)"'])
        self.assertEqual(parser.phases[10].statement_list, ['rlRun "abrt-auto-reporting disabled"',
                                                            'get_configured_value', 'rlAssertEquals "Saves the configuration" "_disabled" "_$CONF_VALUE"',
                                                            'rlAssertEquals "Reads the configuration" "_disabled" "_$(abrt-auto-reporting)"'])
        self.assertEqual(parser.phases[12].statement_list, ['rlRun "abrt-auto-reporting enabled"',
                                                            'get_configured_value', 'rlAssertEquals "Saves the configuration" "_enabled" "_$CONF_VALUE"',
                                                            'rlAssertEquals "Reads the configuration" "_enabled" "_$(abrt-auto-reporting)"'])
        self.assertEqual(parser.phases[14].statement_list[0], 'OLD="enabled"')
        self.assertEqual(parser.phases[14].get_statement_list(), ['OLD="enabled"', 'for arg in disabled EnAbLeD '
                                                                                   'dIsAblEd enabled no Yes nO yes 0 1',
                                                                  'do', 'rlRun "abrt-auto-reporting $arg"#@',
                                                                  'get_configured_value #@', 'rlAssertNotEquals '
                                                                                             '"Changed the '
                                                                                             'configuration" '
                                                                                             '"_$OLD" "_$CONF_VALUE"',
                                                                  'if [ $CONF_VALUE != "enabled" ] && [ $CONF_VALUE !='
                                                                  ' "disabled" ]; then',
                                                                  'rlFail "Mangles the configuration value"#@',
                                                                  'OLD=$CONF_VALUE #@'])
        self.assertEqual(parser.phases[16].statement_list, ['rlRun "abrt-auto-reporting disabled"', 'popd # TmpDir',
                                                            'rm -rf $TmpDir'])

    def test_phases_comments(self):
        generator = Generator("./examples/markup/abrt-auto-reporting-sanity-test.sh")
        generator.parse_file()
        parser = generator.parser_ref
        self.assertEqual(parser.phases[0].get_comments_list(), [[['#@', '@description', 'does', 'what', 'it', 'does']], [['#@', 'Function', 'that', 'configure', 'selected', 'value'], ['#@', 'does', 'not', 'require', 'any', 'parameter']], [['#@', 'comment', 'on', 'the', 'beggining']], [['#@', 'Important', 'case']], [['#@', 'comment', 'on', 'the', 'end']], [['#@', '@author', 'Janosik', 'Karel']]])
        self.assertEqual(parser.phases[1].get_comments_list(), [[['#@', 'Somenthing', 'in', 'start', 'of', 'the', 'test'], ['#Could', 'be', 'anything']]])
        self.assertEqual(parser.phases[2].get_comments_list(), [[['#@', 'Make', 'temporary', 'directory', 'and', 'saves', 'work', 'in', 'it']]])
        self.assertEqual(parser.phases[3].get_comments_list(), [[['#@', 'Additional', 'info']]])
        self.assertEqual(parser.phases[4].get_comments_list(), [[['#', '@', 'Print', 'help', 'informations'], ['#', '@keywords', 'doc', 'help'], ['#', 'blah', 'sfsdf', 'sdf', 'sdfsd', 'f'], ['#', 'Using', 'of', 'block', 'comment'], ['#', 'This', 'could', 'be', 'usefull']], [['#@', 'TESTING']], [['#@', 'Back', 'phase', 'testing']]])
        self.assertEqual(parser.phases[5].get_comments_list(), [])
        self.assertEqual(parser.phases[6].get_comments_list(), [[['#@', 'Runs', 'script', 'with', 'no', 'arguments']]])
        self.assertEqual(parser.phases[7].get_comments_list(), [])
        self.assertEqual(parser.phases[8].get_comments_list(), [[['#@Single', 'enablned', 'as', 'a', 'argument']]])
        self.assertEqual(parser.phases[9].get_comments_list(), [])
        self.assertEqual(parser.phases[10].get_comments_list(), [[['#@', 'Disabled', 'as', 'a', 'argument']], [['#@@key', 'mamut']]])
        self.assertEqual(parser.phases[11].get_comments_list(), [])
        self.assertEqual(parser.phases[12].get_comments_list(), [[['#@', 'More', 'than', 'just', 'one', 'enabled', 'as', 'a', 'argument']]])
        self.assertEqual(parser.phases[13].get_comments_list(), [])
        self.assertEqual(parser.phases[14].get_comments_list(), [[['#@', 'Various', 'types', 'of', 'arguments', 'will', 'start', 'this', 'part']], [['#@', 'for', 'every', 'argument', 'in', 'selected', 'word', 'will', 'do...']], [['']], [['']], [['#@', 'Test', 'if', 'actualy', 'value', 'in', 'arg', 'is', 'not', '"enabled"', 'and', '"disabled"']], [['#@', 'wee', 'will', 'seee']], [['']], [['#@', 'something', 'which', 'is', 'connected', 'to', 'this', 'loop']], [['']], [['#@', 'Documentation', 'test']]])
        self.assertEqual(parser.phases[15].get_comments_list(), [])
        self.assertEqual(parser.phases[16].get_comments_list(), [[['#@', 'clean', 'after', 'test']], [['#@', 'Disable', 'auto', 'reporting']]])
        self.assertEqual(parser.phases[17].get_comments_list(), [[['#@Something', 'on', 'the', 'end', 'of', 'the', 'test'], ['#could', 'be', 'anything']]])
        self.assertEqual(parser.phases[18].get_comments_list(), [])

    def test_conditions(self):
        generator = Generator("./examples/markup/condition_test.sh")
        generator.parse_file()
        parser = generator.parser_ref
        phases = parser.phases
        first_condition = phases[0].comments_list[0]
        self.assertEqual(first_condition.statement_list, ['if [ $? -eq 0 ]', 'then', 'rlReport $1 PASS'])
        self.assertEqual(first_condition.elif_parts[0].statement_list, ['else', 'rlReport $1 FAIL', 'RESULT="FAIL"'])

        second_condition = phases[0].comments_list[1]
        self.assertEqual(second_condition.statement_list, ['if [ $year -eq "0" ]; then', 'echo "This is a leap year.  February has 29 days."'])
        self.assertEqual(second_condition.elif_parts[0].statement_list, ['elif [ $year -eq 0 ]; then', 'echo "This is not a leap year, February has 29 days."'])
        self.assertEqual(second_condition.elif_parts[1].statement_list, ['else', 'echo "This is not a leap year.  February has 28 days."'])

        third_condition = phases[0].comments_list[2]
        self.assertEqual(third_condition.statement_list, ['if [ $year -eq "0" ]; then', 'echo "This is a leap year.  February has 29 days."'])
        self.assertEqual(third_condition.elif_parts[0].statement_list[0], "elif [ $year -eq 0 ]; then")
        self.assertEqual(third_condition.elif_parts[0].statement_list[1].statement_list, ['if [ $year -ne 0 ]; then', 'echo "This is not a leap year, February has 29 days."'])
        self.assertEqual(third_condition.elif_parts[0].statement_list[1].elif_parts[0].statement_list, ['else', 'echo "This is a leap year.  February has 28 days."'])
        self.assertEqual(third_condition.elif_parts[1].statement_list, ['else', 'echo "This is not a leap year.  February has 28 days."'])
        # print phases[0].comments_list
        self.assertEqual(phases[0].get_statement_list(),['rlReport $1 FAIL', 'if [ $? -eq 0 ]', 'then', 'rlReport $1 PASS', 'else', 'rlReport $1 FAIL', 'RESULT="FAIL"', 'if [ $year -eq "0" ]; then', 'echo "This is a leap year.  February has 29 days."', 'elif [ $year -eq 0 ]; then', 'echo "This is not a leap year, February has 29 days."', 'else', 'echo "This is not a leap year.  February has 28 days."', 'if [ $year -eq "0" ]; then', 'echo "This is a leap year.  February has 29 days."', 'elif [ $year -eq 0 ]; then', 'if [ $year -ne 0 ]; then', 'echo "This is not a leap year, February has 29 days."', 'else', 'echo "This is a leap year.  February has 28 days."', 'else', 'echo "This is not a leap year.  February has 28 days."', 'if [ $year -eq "0" ]; then', 'echo "This is a leap year.  February has 29 days."', 'elif [ $year -eq 0 ]; then', 'echo "This is not a leap year, February has 29 days."', 'else', 'echo "This is not a leap year.  February has 28 days."', 'if [ $year -eq "0" ]; then', 'echo "This is a leap year.  February has 29 days."', 'elif [ $year -eq 0 ]; then', 'if [ $year -ne 0 ]; then', 'echo "This is not a leap year, February has 29 days."', 'else', 'echo "This is a leap year.  February has 28 days."', 'else', 'echo "This is not a leap year.  February has 28 days."', 'if [ $year -eq "0" ]; then', 'echo "This is a leap year.  February has 29 days."', 'elif [ $year -eq 0 ]; then', 'echo "This is not a leap year, February has 29 days."', 'else', 'echo "This is not a leap year.  February has 28 days."'])
        self.assertEqual(phases[0].get_comments_list(), [[['#@', 'condition']], [['#@', 'conditionaaaaaaaaaaaa']], [['#@', 'conditionoooooooooooo']], [['#@', 'conditioneeeeeeeeeee']], [['#@', 'conditionesss']], [['#@', 'conditionassss']], [['#@', 'conditionsdaswewqw']], [['#@', 'conditionttttttttttttt']], [['#@', 'Little', 'condition', 'description', 'without', 'inside', 'conditions']], [['#@', 'Big', 'condition', 'description', 'with', 'inside', 'conditions']], [['#@', 'Little', 'condition', 'description', 'without', 'inside', 'conditions']], [['#@', 'Little', 'condition', 'description', 'without', 'inside', 'conditions']], [['#@', 'Little', 'condition', 'description', 'without', 'inside', 'conditions', 'ELIF']]])

    def test_abrt(self):
        generator = Generator("./examples/markup/abrt-auto-reporting-sanity-test.sh")
        generator.parse_file()
        generator.comments_set_up()
        doc = generator.get_documentation()
        first_doc = open("./examples/markup/abrt-PURPOSE.txt", 'r')
        data = first_doc.read()
        self.assertEqual(doc, data)

    def test_condition(self):
        generator = Generator("./examples/markup/condition_test.sh")
        generator.parse_file()
        generator.comments_set_up()
        doc = generator.get_documentation()
        first_doc = open("./examples/markup/condition-PURPOSE.txt", 'r')
        data = first_doc.read()
        self.assertEqual(doc, data)

    def test_containers(self):
        generator = Generator("./examples/markup/containers_tests.sh")
        generator.parse_file()
        generator.comments_set_up()
        doc = generator.get_documentation()
        first_doc = open("./examples/markup/containers-PURPOSE.txt", 'r')
        data = first_doc.read()
        self.assertEqual(doc, data)

    def test_loops(self):
        generator = Generator("./examples/markup/loop_test.sh")
        generator.parse_file()
        generator.comments_set_up()
        doc = generator.get_documentation()
        first_doc = open("./examples/markup/loop-PURPOSE.txt", 'r')
        data = first_doc.read()
        self.assertEqual(doc, data)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
__author__ = 'Jiri_Kulda'
# description: Simple parser for BeakerLib test

import shlex
import sys
import bkrdoc.analysis


class Parser(object):
    """
    Parser is main class. It contains methods for running analysis and nature language information creation.
    Also contains phases containers.
    :param file_in: is path with name to file from which the documentation will be created.
    """
    lexer = shlex

    file_test = ""

    description = ""

    all_commands = ["rlAssert0", "rlAssertEquals", "rlAssertNotEquals",
                    "rlAssertGreater", "rlAssertGreaterOrEqual", "rlAssertExists", "rlAssertNotExists",
                    "rlAssertGrep", "rlAssertNotGrep", "rlAssertDiffer", "rlAssertNotDiffer", "rlRun",
                    "rlWatchdog", "rlReport", "rlIsRHEL", "rlIsFedora", "rlCheckRpm", "rlAssertRpm",
                    "rlAssertNotRpm", "rlAssertBinaryOrigin", "rlGetMakefileRequires",
                    "rlCheckRequirements", "rlCheckMakefileRequires", "rlMount", "rlCheckMount",
                    "rlAssertMount", "rlHash", "rlUnhash", "rlFileBackup", "rlFileRestore",
                    "rlServiceStart", "rlServiceStop", "rlServiceRestore", "rlSEBooleanOn",
                    "rlSEBooleanOff", "rlSEBooleanRestore", "rlCleanupAppend", "rlCleanupPrepend",
                    "rlVirtualXStop", "rlVirtualXGetDisplay", "rlVirtualXStart", "rlWait", "rlWaitForSocket",
                    "rlWaitForFile", "rlWaitForCmd", "rlImport", "rlDejaSum", "rlPerfTime_AvgFromRuns",
                    "rlPerfTime_RunsinTime", "rlLogMetricLow", "rlLogMetricHigh", "rlShowRunningKernel",
                    "rlGetDistroVariant", "rlGetDistroRelease", "rlGetSecondaryArch", "rlGetPrimaryArch",
                    "rlGetArch", "rlShowPackageVersion", "rlFileSubmit", "rlBundleLogs", "rlDie",
                    "rlLogFatal", "rlLogError", "rlLogWarning", "rlLogInfo", "rlLogDebug", "rlLog",
                    "rlGetTestState", "rlGetPhaseState", "rlJournalPrint", "rlJournalPrintText"]

    phases = []
    outside = ""
    file_name = ""

    test_launch = ""

    environmental_variable = []

    def __init__(self, file_in):
        self.phases = []
        self.test_launch = 0
        self.environmental_variable = []
        file_in = file_in.strip()
        if file_in[(len(file_in) - 3):len(file_in)] == ".sh":
            try:
                with open(file_in, "r") as input_file:
                    self.file_name = file_in
                    self.description = file_in[0:(len(file_in) - 3)]
                    self.file_test = input_file.read()
                    self.parse_data()

            except IOError:
                sys.stderr.write("ERROR: Fail to open file: " + file_in + "\n")
                sys.exit(1)

        else:
            print("ERROR: Not a script file. (*.sh)")
            sys.exit(1)

    def parse_data(self):
        """
        Method which divides lines of code into phase containers.
        """
        self.phases.append(bkrdoc.analysis.PhaseOutside())

        pom_line = ""
        for line in self.file_test.split('\n'):
            line = line.strip()

            if line[0:1] != '#' and len(line) >= 1 and \
                    not self.is_phase_journal_end(line):

                if self.is_phase(line):
                    self.phases.append(bkrdoc.analysis.PhaseContainer(line[len("rlphasestart"):]))

                elif self.is_end_back_slash(line):
                    pom_line += line[0:-1]

                elif len(self.phases) > 0:
                    if pom_line != "":
                        self.phases[-1].setup_statement(pom_line + line)
                        pom_line = ""
                    else:
                        self.phases[-1].setup_statement(line)

            elif self.is_phase_journal_end(line):
                self.phases.append(bkrdoc.analysis.PhaseOutside())

    def print_statement(self):
        for i in self.phases:
            print(i.statement_list)
            print("\n")

    def is_end_back_slash(self, line):
        return line[-1:] == '\\'

    def is_phase(self, line):
        return line[0:len("rlphasestart")].lower() == "rlphasestart"

    def is_phase_clean(self, line):
        return line[0:len("rlphasestartclean")].lower() == "rlphasestartclean"

    def is_phase_test(self, line):
        return line[0:len("rlphasestarttest")].lower() == "rlphasestarttest"

    def is_phase_setup(self, line):
        return line[0:len("rlphasestartsetup")].lower() == "rlphasestartsetup"

    def is_phase_journal_end(self, line):
        if line[0:len("rlphaseend")].lower() == "rlphaseend":
            return True

        elif line[0:len("rljournalend")].lower() == "rljournalend":
            return True

        else:
            return False

    def is_journal_start(self, line):
        return line[0:len("rljournalstart")].lower() == "rljournalstart"

    def is_phase_outside(self, phase_ref):
        return phase_ref.phase_name == "Outside phase"

    def is_beakerlib_command(self, testing_command):
        return testing_command in self.all_commands

    def get_phases(self):
        return self.phases

    def get_environmental_variables(self):
        return self.environmental_variable

    def get_file_name(self):
        return self.file_name

    def get_test_launch(self):
        return self.test_launch

    def set_test_launch(self, number_of_variable):
        self.test_launch = number_of_variable


#!/usr/bin/python
import argparse
import re
import shlex
import bkrdoc.analysis

__author__ = 'Jiri_Kulda'

class StatementDataSearcher:
    """
    This class is responsible for parsing data from statement lines. This parsing is done by
    setting argparse modules for every BeakerLib command. These setting we can see under
    big switch.
    :param generator_ref: parser reference
    :param phase_ref: reference to phase where was StatementDataSearcher instance made.
    """
    parsed_param_ref = ""
    generator_ref = ""
    phase_ref = ""

    minimum_variable_size = 4

    def __init__(self, generator_ref, phase_ref):
        self.generator_ref = generator_ref
        self.phase_ref = phase_ref
        self.minimum_variable_size = 4

    def parse_command(self, statement_line):
        # Splitting statement using shlex lexicator
        """
        Method contains big switch for division of statement line
        :param statement_line: singe line of code from test
        :return: argparse object with parsed data
        """
        pom_statement_line = self.phase_ref.variables.replace_variable_in_string(statement_line)
        self.get_cmd_line_params(pom_statement_line)
        self.get_environmental_variable(pom_statement_line)
        pom_list = shlex.split(pom_statement_line, True, posix=True)
        first = pom_list[0]

        # if self.is_beakerLib_command(first, self.parser_ref):
        condition = bkrdoc.analysis.ConditionsForCommands()

        if condition.is_rlrun_command(first):
            self.get_rlrun_data(pom_list)

        elif condition.is_rpm_command(first):
            self.get_rpmcommand_data(pom_list)

        elif condition.is_check_or_assert_mount(first):
            self.get_check_or_assert_mount_data(pom_list)

        elif condition.is_assert_command(first):

            if condition.is_assert_grep(first):
                self.get_assert_grep_data(pom_list)

            elif condition.is_rlpass_or_rlfail_command(first):
                self.get_rlpass_or_rlfail_data(pom_list)

            elif condition.is_assert0(first):
                self.get_assert0_data(pom_list)

            elif condition.is_assert_comparasion(first):
                self.get_assert_comparison_data(pom_list)

            elif condition.is_assert_exists(first):
                self.get_assert_exits_data(pom_list)

            elif condition.is_assert_differ(first):
                self.get_assert_differ_data(pom_list)

            elif condition.is_assert_binary_origin(first):
                self.get_assertbinaryorigin_data(pom_list)

        elif condition.is_rlfilebackup_command(first):
            self.get_rlfilebackup_data(pom_list)

        elif condition.is_rlfilerestore_command(first):
            self.get_rlfile_restore_data(pom_list)

        elif condition.is_rlisrhel_or_rlisfedora_command(first):
            self.get_isrhel_or_isfedora_data(pom_list)

        elif condition.is_rlmount(first):
            self.get_rlmount_data(pom_list)

        elif condition.is_rlhash_or_rlunhash_command(first):
            self.get_rlhash_or_rlunhash_data(pom_list)

        elif condition.is_rllog_command(first):
            self.get_rllog_data(pom_list)

        elif condition.is_rldie_command(first):
            self.get_rldie_data(pom_list)

        elif condition.is_rlget_x_arch_command(first):
            self.get_rlget_commands_data(pom_list)

        elif condition.is_rlgetdistro_command(first):
            self.get_rlget_commands_data(pom_list)

        elif condition.is_rlgetphase_or_test_state_command(first):
            self.get_rlget_commands_data(pom_list)

        elif condition.is_rlreport_command(first):
            self.get_rlreport_data(pom_list)

        elif condition.is_rlwatchdog_command(first):
            self.get_rlwatchdog_data(pom_list)

        elif condition.is_rlbundlelogs_command(first):
            self.get_rlbundlelogs_data(pom_list)

        elif condition.is_rlservicexxx(first):
            self.get_rlservicexxx_data(pom_list)

        elif condition.is_sebooleanxxx_command(first):
            self.get_sebooleanxxx_data(pom_list)

        elif condition.is_rlshowrunningkernel_command(first):
            self.get_rlshowrunningkernel_data(pom_list)

        elif condition.is_get_or_check_makefile_requires(first):
            self.get_rlget_or_rlcheck_makefilerequeries_data(pom_list)

        elif condition.is_rlcleanup_apend_or_prepend_command(first):
            self.get_rlcleanup_apend_or_prepend_data(pom_list)

        elif condition.is_rlfilesubmit_command(first):
            self.get_rlfilesubmit_data(pom_list)

        elif condition.is_rlperftime_runsintime_command(first):
            self.get_rlperftime_runsintime_data(pom_list)

        elif condition.is_rlperftime_avgfromruns_command(first):
            self.get_rlperftime_avgfromruns_data(pom_list)

        elif condition.is_rlshowpackageversion_command(first):
            self.get_rlshowpackageversion_data(pom_list)

        elif condition.is_rljournalprint_command(first):
            self.get_rljournalprint_data(pom_list)

        elif condition.is_rlimport_command(first):
            self.get_rlimport_data(pom_list)

        elif condition.is_rlwaitforxxx_command(first):
            self.get_rlwaitforxxx_data(pom_list, first)

        elif condition.is_rlwaitfor_command(first):
            self.get_rlwaitfor_data(pom_list)

        elif condition.is_virtualxxx_command(first):
            self.get_rlvirtualx_xxx_data(pom_list)

        else:
            self.unknown_command(pom_list, pom_statement_line)

        return self.parsed_param_ref

    def find_and_replace_variable(self, statement):
        pass

    def get_cmd_line_params(self, line):
        """
        This method searches for command line variables in code represented as $1 $2 ...
        :param line: statement line of code
        """
        regular = re.compile("(.*)(\$(\d+))(.*)")
        match = regular.match(line)
        if match:
            self.generator_ref.set_test_launch(match.group(3))

    def get_environmental_variable(self, line):
        """
        Searches environmental variables in code line
        :param line: code line
        """
        lexer = shlex.shlex(line)
        word = lexer.get_token()
        while word:
            if word == "$":
                word = lexer.get_token()
                if not self.phase_ref.variables.is_existing_variable(word) and len(word) > self.minimum_variable_size:
                    self.generator_ref.set_environmental_variable_information(word)

            elif word[0:1] == '"':  # shlex doesn't returns whole string so for searching in strings I'm using recursion
                self.get_environmental_variable(word[1:-1])
            word = lexer.get_token()

    def is_variable_assignment(self, statement):
        """
        searching variables in statement line
        :param statement: code line
        :return: returns nothing
        """
        read = shlex.shlex(statement)
        member = read.get_token()
        equal_to = read.get_token()
        while equal_to:
            # condition to handle assign to random value
            # setting variable list
            if equal_to == '=':
                # This 7 lines are here for erasing comments and for reading whole line
                pom_i = statement.find("=", len(member)) + 1
                list_of_statement = shlex.split(statement[pom_i:], True, True)
                value = ""
                for value_member in list_of_statement:
                    if not value == "":
                        value += " "
                    value += value_member

                regular = re.compile("\"(/.*/)(.*)\"")
                match = regular.match(value)
                if match:
                    self.phase_ref.variables.add_variable(member, match.group(1) + match.group(2))
                    # TODO keywords from not outside phases
                    # self.keywords_list.append(match.group(2))
                else:
                    self.phase_ref.variables.add_variable(member, value)

            member = equal_to
            equal_to = read.get_token()

        return

    def get_rljournalprint_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("type", type=str, nargs="?")
        parser_arg.add_argument('--full-journal', dest='full_journal',
                                action='store_true', default=False)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlshowpackageversion_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("package", type=str, nargs="+")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlfilesubmit_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("-s", type=str, help="sets separator")
        parser_arg.add_argument("path_to_file", type=str)
        parser_arg.add_argument("required_name", type=str, nargs="?")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlbundlelogs_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("package", type=str)
        parser_arg.add_argument("file", type=str, nargs="+")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rldie_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("message", type=str)
        parser_arg.add_argument("file", type=str, nargs="*")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rllog_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("message", type=str)
        parser_arg.add_argument("logfile", type=str, nargs="?")
        parser_arg.add_argument("priority", type=str, nargs="?")
        parser_arg.add_argument('--prio-label', dest='prio_label',
                                action='store_true', default=False)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlshowrunningkernel_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlget_or_rlcheck_makefilerequeries_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlget_commands_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def unknown_command(self, pom_param_list, statement_list):
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        self.parsed_param_ref = parser_arg.parse_args(["UNKNOWN"])
        # Trying to find variable assignment in statement line
        self.is_variable_assignment(statement_list)
        self.is_function_name_in_statement(statement_list)

    def is_function_name_in_statement(self, line):
        for function in self.phase_ref.get_function_list():
            if function.name in line and function.is_function_data_empty():
                self.phase_ref.search_data_in_function(function)

    def get_rlwatchdog_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("command", type=str)
        parser_arg.add_argument("timeout", type=str)
        parser_arg.add_argument("signal", type=str, nargs='?', default="KILL")
        parser_arg.add_argument("callback", type=str, nargs='?')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlreport_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("name", type=str)
        parser_arg.add_argument("result", type=str)
        parser_arg.add_argument("score", type=str, nargs='?')
        parser_arg.add_argument("log", type=str, nargs='?')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlrun_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('-t', dest='t', action='store_true', default=False)
        parser_arg.add_argument('-l', dest='l', action='store_true', default=False)
        parser_arg.add_argument('-c', dest='c', action='store_true', default=False)
        parser_arg.add_argument('-s', dest='s', action='store_true', default=False)
        parser_arg.add_argument("command", type=str)
        parser_arg.add_argument("status", type=str, nargs='?', default="0")
        parser_arg.add_argument("comment", type=str, nargs='?')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)
        ref = self.parsed_param_ref
        self.parse_command(self.parsed_param_ref.command)  # for getting variables from command
        self.parsed_param_ref = ref

    def get_rlvirtualx_xxx_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("name", type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlwaitfor_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('n', type=str, nargs='*')
        parser_arg.add_argument("-t", type=int, help="time")
        parser_arg.add_argument("-s", type=str, help="SIGNAL", default="SIGTERM")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlwaitforxxx_data(self, pom_param_list, command):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        :param command: command name
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("-p", type=str, help="PID")
        parser_arg.add_argument("-t", type=str, help="time")
        parser_arg.add_argument("-d", type=int, help="delay", default=1)

        if bkrdoc.analysis.ConditionsForCommands().is_rlwaitforcmd_command(command):
            parser_arg.add_argument("command", type=str)
            parser_arg.add_argument("-m", type=str, help="count")
            parser_arg.add_argument("-r", type=str, help="retrval", default="0")

        elif bkrdoc.analysis.ConditionsForCommands().is_rlwaitforfile_command(command):
            parser_arg.add_argument("path", type=str)

        elif bkrdoc.analysis.ConditionsForCommands().is_rlwaitforsocket_command(command):
            parser_arg.add_argument("port_path", type=str)
            parser_arg.add_argument('--close', dest='close', action='store_true',
                                    default=False)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlimport_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("LIBRARY", type=str, nargs='+')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlperftime_runsintime_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("command", type=str)
        parser_arg.add_argument("time", type=int, nargs='?', default=30)
        parser_arg.add_argument("runs", type=int, nargs='?', default=3)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlperftime_avgfromruns_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("command", type=str)
        parser_arg.add_argument("count", type=int, nargs='?', default=3)
        parser_arg.add_argument("warmup", type=str, nargs='?', default="warmup")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlcleanup_apend_or_prepend_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("string", type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_sebooleanxxx_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("boolean", type=str, nargs='+')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlservicexxx_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("service", type=str, nargs='+')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlfile_restore_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument("--namespace", type=str,
                                help="specified namespace to use")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlfilebackup_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('--clean', dest='clean', action='store_true',
                                default=False)
        parser_arg.add_argument("--namespace", type=str,
                                help="specified namespace to use")
        parser_arg.add_argument('file', type=str, nargs='+')
        parser_arg.add_argument('status', type=str, nargs='?', default="-")
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlhash_or_rlunhash_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('--decode', dest='decode', action='store_true',
                                default=False, help='unhash given string')
        parser_arg.add_argument("--algorithm", type=str,
                                help="given hash algorithm")
        parser_arg.add_argument("STRING", type=str, nargs='?')
        parser_arg.add_argument('--stdin', action='store_true', default=False)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_check_or_assert_mount_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('server', type=str, nargs='?')
        parser_arg.add_argument('share', type=str, nargs='?')
        parser_arg.add_argument('mountpoint', type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlmount_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('server', type=str)
        parser_arg.add_argument('share', type=str)
        parser_arg.add_argument('mountpoint', type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_assertbinaryorigin_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('binary', type=str)
        parser_arg.add_argument('package', type=str, nargs='*')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rpmcommand_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        if len(pom_param_list) == 2 and pom_param_list[1] == "--all":
            parser_arg.add_argument('--all', dest='all', action='store_true',
                                    default=False, help='assert all packages')
            self.parsed_param_ref = parser_arg.parse_args(pom_param_list)
        else:
            parser_arg.add_argument('name', type=str)
            parser_arg.add_argument('version', type=str, nargs='?')
            parser_arg.add_argument('release', type=str, nargs='?')
            parser_arg.add_argument('arch', type=str, nargs='?')
            # this line is for information translator
            parser_arg.add_argument('--all', dest='all', action='store_true',
                                    default=False, help='assert all packages')
            self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_isrhel_or_isfedora_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('type', type=str, nargs='*')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_assert_differ_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('file1', type=str)
        parser_arg.add_argument('file2', type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_assert_exits_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('file_directory', type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_assert_comparison_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('comment', type=str)
        parser_arg.add_argument('value1', type=str)
        parser_arg.add_argument('value2', type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_assert0_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('comment', type=str)
        parser_arg.add_argument('value', type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_rlpass_or_rlfail_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('comment', type=str)
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def get_assert_grep_data(self, pom_param_list):
        """
        Parsing data from statement line using set upped argparse module
        :param pom_param_list: code line
        """
        parser_arg = argparse.ArgumentParser()
        parser_arg.add_argument("argname", type=str)
        parser_arg.add_argument('pattern', type=str)
        parser_arg.add_argument('file', type=str)
        parser_arg.add_argument('-i', '-I', dest='text_in', action='store_true',
                                default=False, help='insensitive matches')
        parser_arg.add_argument('-e', '-E', dest='moin_in', action='store_true',
                                default=False, help='Extended grep')
        parser_arg.add_argument('-p', '-P', dest='out_in', action='store_true',
                                default=False, help='perl regular expression')
        self.parsed_param_ref = parser_arg.parse_args(pom_param_list)

    def is_beakerlib_command(self, testing_command, parser_ref):
        return parser_ref.is_beakerlib_command(testing_command)


#!/usr/bin/python
# author Jiri Kulda

from __future__ import division  # floating-point division

import argparse
import bkrdoc.analysis


class DocumentationGenerator:

    _parser_ref = ""
    _phases = ""

    def __init__(self):
        self._parser_ref = ""
        self._phases = ""

    def parse_given_file(self, file):
            self._parser_ref = bkrdoc.analysis.Parser(file)
            self._phases = self._parser_ref.get_phases()

    def set_test_launch(self, number_of_variables):
        """
        This method sets up test launch variable, which indicates number of variables.
        :param number_of_variables: number of command line variables in test
        """
        if int(number_of_variables) > int(self._parser_ref.get_test_launch()):
            self._parser_ref.set_test_launch(number_of_variables)

    def set_environmental_variable_information(self, variable):
        """
        This method sets up test environmental variable information command. This command
        contains names of environmental variables in test.
        :param variable: founded environmental variable in test.
        """
        if variable not in self._parser_ref.environmental_variable:
            self._parser_ref.environmental_variable.append(variable)

    def get_doc_data(self):
        """
        Method which is responsible for starting the first analysis of code lines.
        After this method finishes all phase containers will have argparse object
        with parsed data.
        """
        pom_var = bkrdoc.analysis.TestVariables()
        pom_func = []
        for member in self._phases:
            if not self.is_phase_outside(member):
                member.search_data(self, pom_var, pom_func)
                pom_var = bkrdoc.analysis.TestVariables()

            else:
                member.search_data(pom_var, pom_func)
                pom_var = bkrdoc.analysis.TestVariables()

            # copying variables to new variable instance
            for var in member.variables.variable_names_list:
                pom_value = member.variables.get_variable_value(var)
                pom_var.add_variable(var, pom_value)

            # copying keywords to new variable instance
            for key in member.variables.keywords:
                pom_var.add_keyword(key)

            # copying functions to new function list
            copy_func_list = member.func_list
            pom_func = copy_func_list

    def get_documentation_information(self):
        """
        This method is responsible for starting of replacement of argparse object to
        DocumentationInformation object.
        """
        for member in self._phases:
            if not self.is_phase_outside(member):
                member.translate_data(self)

    def generate_documentation(self):
        """
        This method starts transformation from DocumentationInformation objects
        to small object with nature language information
        """
        for member in self._phases:
            if not self.is_phase_outside(member):
                member.generate_documentation()

    def get_test_weigh(self):
        """
        This method calculates number of lines in test.

        :return: number of lines where information will be placed.
        """
        weigh = 0
        for member in self._phases:
            if not self.is_phase_outside(member):
                weigh += member.get_phase_weigh()
        return weigh

    def setup_phases_lists_for_knapsack(self):
        """
        This method sets up list for knapsack algorithm.

        :return: Set up list.
        """
        phases_list = self.get_phases_information_lists()
        whole_set_uped_list = []
        for element in phases_list:
            pom_list = [element, element.get_information_weigh(), element.get_information_value()]
            whole_set_uped_list.append(pom_list)
        return whole_set_uped_list

    def get_phases_information_lists(self):
        """
        This method gathers all information lists in phase containers into big one

        :return: one big information list
        """
        phases_lists = []
        for member in self._phases:
            if not self.is_phase_outside(member):
                phases_lists += member.get_information_list()
        return phases_lists

    def set_phases_information_lists(self, finished_knapsack_list):
        """
        This method checks number of lines in phases containers. If the lines are
        bigger then this method will run prepared knapsack algorithm.
        :param finished_knapsack_list: final list with information
        """
        pom_phases_lists = []
        for member in self._phases:
            if not self.is_phase_outside(member):
                pom = member.get_information_list()
                for element in finished_knapsack_list:
                    if element[0] in pom:
                        pom_phases_lists.insert(0, element[0])
                member.set_information_list(pom_phases_lists)
                pom_phases_lists = []

    def print_test_launch(self):
        """
        This method prints test launch information
        """
        inf = "Test launch: " + str(self._parser_ref.get_file_name())
        i = 0
        while int(i) < int(self._parser_ref.get_test_launch()):
            inf += " [VARIABLE]"
            i += 1
        print(inf)

    def print_test_environmental_variables_information(self):
        """
        This method prints environmental variables information
        """
        inf = "Test environmental variables: "
        if len(self._parser_ref.get_environmental_variables()):
            for env in self._parser_ref.get_environmental_variables():
                inf += env + ", "
            inf = inf[0:-2]
        else:
            inf += "-"
        print(inf)

    def print_documentation(self, cmd_options):
        """
        This method is responsible for printing the final documentation to stdout
        :param cmd_options: possible command line options
        """
        self.print_test_launch()
        self.print_test_environmental_variables_information()
        print("")
        test_weigh = self.get_test_weigh()

        if not cmd_options.print_all and test_weigh > cmd_options.size:
            knapsack_list = self.setup_phases_lists_for_knapsack()
            finished_knapsack = self.solve_knapsack_dp(knapsack_list, cmd_options.size)
            self.set_phases_information_lists(finished_knapsack)

        for member in self._phases:
            if not self.is_phase_outside(member):
                member.print_phase_documentation(cmd_options)
                print("")

        print("[ Overall documentation credibility: " + self.get_overall_credibility() + " ]")

    # items in [["information", weigh, value], ...] format
    def solve_knapsack_dp(self, items, limit):
        """
        Algorithm for solving the knapsack problem
        :param items: set up list of information
        :param limit: specified size limit
        :return: final list with information with best value
        """
        global xrange
        try:
            xrange
        except NameError:
            xrange = range

        table = [[0 for w in range(limit + 1)] for j in xrange(len(items) + 1)]  # initialization of table
        for j in xrange(1, len(items) + 1):
            item, wt, val = items[j-1]
            for w in xrange(1, limit + 1):
                if wt > w:
                    table[j][w] = table[j-1][w]
                else:
                    table[j][w] = max(table[j-1][w],
                                      table[j-1][w-wt] + val)

        result = []
        w = limit
        for j in range(len(items), 0, -1):
            was_added = table[j][w] != table[j-1][w]

            if was_added:
                item, wt, val = items[j-1]
                result.append(items[j-1])
                w -= wt
        return result

    def search_variable(self, phase_ref, searching_variable):
        """
        this method searches specified test variable
        :param phase_ref: reference to phase container
        :param searching_variable: <-
        """
        pom_variable = ""

        for member in self._phases:
            if self.is_phase_outside(member):
                pom_pos = member.get_variable_position(searching_variable)
                if pom_pos >= 0:
                    pom_variable = member.variable_values_list[pom_pos]

            elif member == phase_ref:
                if pom_variable == "":
                    print("UNKNOWN VARIABLE !!!")
                return pom_variable

    def is_phase_outside(self, phase_ref):
        return phase_ref.phase_name == "Outside phase"

    def get_overall_credibility(self):
        count = 0
        overall_percent_correct = 0.0
        for phase in self._phases:
            if not self.is_phase_outside(phase):
                overall_percent_correct += phase.get_phase_credibility().get_percent_correct()
                count += 1
        if count == 0:
            return 'Very high'
        else:
            return bkrdoc.analysis.credibility.DocumentationCredibility(overall_percent_correct/count).get_credibility()

#  ***************** MAIN ******************
def set_cmd_arguments():
    """
    This method contains set upped argparse object to parse
    command line options and files

    :return: argparse object
    """
    pom_parser = argparse.ArgumentParser(description='Parse arguments in cmd line for generator')
    pom_parser.add_argument('files', metavar='file', type=str, nargs='+', help='script file')
    pom_parser.add_argument('-l', '--log', dest='log_in', action='store_true',
                            default=False, help='Show log data if they are possible')
    pom_parser.add_argument('-s', '--size', type=int, help="Size of output documentation in lines,"
                                                           " default is 32 lines(A4) per documentation", default=32)
    pom_parser.add_argument('--all', '--print-all', dest='print_all', action='store_true',
                            default=False, help='Printing all possible data.')
    parser_arg = pom_parser.parse_args()
    return parser_arg


def run_analysis_doc_generator(parser_arg):
    """
    This method runs documentation generator
    :param parser_arg: argparse object
    """
    for one_file in parser_arg.files:
        doc_generator = DocumentationGenerator()
        doc_generator.parse_given_file(one_file)
        doc_generator.get_doc_data()
        doc_generator.get_documentation_information()
        doc_generator.generate_documentation()
        doc_generator.print_documentation(parser_arg)

if __name__ == "__main__":
    CMD_args = set_cmd_arguments()
    run_analysis_doc_generator(CMD_args)



#!/usr/bin/python
__author__ = 'Jiri_Kulda'

class Topic(object):
    """
    Class which is consist of information topic data.
    For example for BeakerLib command rlRun is topic command.

    :param topic_data: topic data (Facts)
    :param subject: Subject data which are "connected" to topic_data
    """
    topic = ""

    subject = []

    def __init__(self, topic_data, subject):
        self.topic = topic_data
        self.subject = subject

    def get_topic(self):
        return self.topic

    def get_subject(self):
        return self.subject


#!/usr/bin/python
__author__ = 'Jiri_Kulda'


class ConditionsForCommands:
    """ Class consists of conditions for testing commands used in
    StatementDataSearcher and DocumentationTranslator """

    def is_rlwatchdog_command(self, command):
        return command == "rlWatchdog"

    def is_rlreport_command(self, command):
        return command == "rlReport"

    def is_virtualxxx_command(self, command):
        pom_list = ["rlVirtualXStop", "rlVirtualXStart", "rlVirtualXGetDisplay"]
        return command in pom_list

    def is_rlwaitfor_command(self, command):
        return command == "rlWaitFor"

    def is_rlwaitforsocket_command(self, command):
        return command == "rlWaitForSocket"

    def is_rlwaitforfile_command(self, command):
        return command == "rlWaitForFile"

    def is_rlwaitforcmd_command(self, command):
        return command == "rlWaitForCmd"

    def is_rlwaitforxxx_command(self, command):
        pom_list = ["rlWaitForCmd", "rlWaitForFile", "rlWaitForSocket"]
        return command in pom_list

    def is_rlimport_command(self, command):
        return command == "rlImport"

    def is_rlperftime_runsintime_command(self, command):
        return command == "rlPerfTime_RunsInTime"

    def is_rlperftime_avgfromruns_command(self, command):
        return command == "rlPerfTime_AvgFromRuns"

    def is_rlcleanup_apend_or_prepend_command(self, command):
        return command == "rlCleanupAppend" or command == "rlCleanupPrepend"

    def is_sebooleanxxx_command(self, command):
        pom_list = ["rlSEBooleanOn", "rlSEBooleanOff", "rlSEBooleanRestore"]
        return command in pom_list

    def is_rlservicexxx(self, command):
        pom_list = ["rlServiceStart", "rlServiceStop", "rlServiceRestore"]
        return command in pom_list

    def is_rlfilebackup_command(self, command):
        return command == "rlFileBackup"

    def is_rlfilerestore_command(self, command):
        return command == "rlFileRestore"

    def is_rlhash_or_rlunhash_command(self, command):
        return command == "rlHash" or command == "rlUnhash"

    def is_check_or_assert_mount(self, command):
        return command == "rlCheckMount" or command == "rlAssertMount"

    def is_get_or_check_makefile_requires(self, command):
        return command == "rlCheckMakefileRequires" or command == "rlGetMakefileRequires"

    def is_rlmount(self, command):
        return command == "rlMount"

    def is_assert_binary_origin(self, command):
        return command == "rlAssertBinaryOrigin"

    def is_rlisrhel_or_rlisfedora_command(self, command):
        return command == "rlIsRHEL" or command == "rlIsFedora"

    def is_assert_differ(self, command):
        return command == "rlAssertDiffer" or command == "rlAssertNotDiffer"

    def is_assert_exists(self, command):
        return command == "rlAssertExists" or command == "rlAssertNotExists"

    def is_assert_comparasion(self, command):
        pom_list = ["rlAssertEquals", "rlAssertNotEquals", "rlAssertGreater",
                    "rlAssertGreaterOrEqual"]
        return command in pom_list

    def is_rlpass_or_rlfail_command(self, command):
        return command == "rlPass" or command == "rlFail"

    def is_assert_grep(self, command):
        return command == "rlAssertGrep" or command == "rlAssertNotGrep"

    def is_assert0(self, command):
        return command == "rlAssert0"

    def is_assert_command(self, line):
        return line[0:len("rlAssert")] == "rlAssert"

    def is_rpm_command(self, command):
        return command[-3:] == "Rpm"

    def is_rlrun_command(self, line):
        return line[0:len("rlRun")] == "rlRun"

    def is_rljournalprint_command(self, command):
        pom_list = ["rlJournalPrint", "rlJournalPrintText"]
        return command in pom_list

    def is_rlgetphase_or_test_state_command(self, command):
        pom_list = ["rlGetPhaseState", "rlGetTestState"]
        return command in pom_list

    def is_rllog_command(self, command):
        pom_list = ["rlLogFatal", "rlLogError", "rlLogWarning", "rlLogInfo",
                    "rlLogDebug", "rlLog"]
        return command in pom_list

    def is_rllogmetric_command(self, command):
        pom_list = ["rlLogMetricLow", "rlLogMetricHigh"]
        return command in pom_list

    def is_rldie_command(self, command):
        return command[0:len("rlDie")] == "rlDie"

    def is_rlbundlelogs_command(self, command):
        return command[0:len("rlBundleLogs")] == "rlBundleLogs"

    def is_rlfilesubmit_command(self, command):
        return command[0:len("rlFileSubmit")] == "rlFileSubmit"

    def is_rlshowpackageversion_command(self, command):
        return command[0:len("rlShowPackageVersion")] == "rlShowPackageVersion"

    def is_rlget_x_arch_command(self, command):
        pom_list = ["rlGetArch", "rlGetPrimaryArch", "rlGetSecondaryArch"]
        return command in pom_list

    def is_rlgetdistro_command(self, command):
        pom_list = ["rlGetDistroRelease", "rlGetDistroVariant"]
        return command in pom_list

    def is_rlshowrunningkernel_command(self, command):
        return command[0:len("rlShowRunningKernel")] == "rlShowRunningKernel"
